@@ -35,19 +35,11 @@ class HTTPRequestBase(ABC):
     @abstractmethod
     def request(self, route: str, meth: str, body: typing.Any = None, *, is_json: bool = False, reason_header: str = None, retry: int = 3, **kwargs):
         """
-        This function should wrap :meth:`._request` with rate limit handling.
+        This function includes requesting REST API.
         :return:
         """
         pass
 
-    @abstractmethod
-    def _request(self, route: str, meth: str, body: dict = None, is_json: bool = False, reason_header: str = None, **kwargs) -> typing.Tuple[int, dict]:
-        """
-        HTTP Requesting part.
-        :return: Tuple[int, dict]
-        """
-        pass
-
     # Channel Requests
 
     def request_channel(self, channel_id):
@@ -806,7 +798,7 @@ class HTTPRequestBase(ABC):
         :param guild_id: ID of the guild to request.
         :param with_counts: Whether to return member and presence count.
         """
-        return self.request(f"/guilds/{guild_id}", "GET", params={"with_counts": with_counts})
+        return self.request(f"/guilds/{guild_id}", "GET", params={"with_counts": "true" if with_counts else "false"})
 
     def request_guild_preview(self, guild_id):
         """
@@ -1434,9 +1426,9 @@ class HTTPRequestBase(ABC):
         """
         params = {}
         if with_counts is not None:
-            params["with_counts"] = with_counts
+            params["with_counts"] = "true" if with_counts else "false"
         if with_expiration is not None:
-            params["with_expiration"] = with_expiration
+            params["with_expiration"] = "true" if with_expiration else "false"
         return self.request(f"/invites/{invite_code}", "GET", params=params)
 
     def delete_invite(self, invite_code, reason: str = None):
@@ -1448,6 +1440,118 @@ class HTTPRequestBase(ABC):
         """
         return self.request(f"/invites/{invite_code}", "DELETE", reason_header=reason)
 
+    # Stage Instance requests
+
+    def create_stage_instance(self, channel_id: str, topic: str, privacy_level: int = None, reason: str = None):
+        """
+        Sends create stage instance request.
+
+        :param channel_id: ID of the stage channel.
+        :param topic: Topic of the stage instance.
+        :param privacy_level: Privacy level of the stage.
+        :param reason: Reason of the action.
+        """
+        body = {"channel_id": channel_id, "topic": topic}
+        if privacy_level is not None:
+            body["privacy_level"] = privacy_level
+        return self.request("/stage-instances", "POST", body, is_json=True, reason_header=reason)
+
+    def request_stage_instance(self, channel_id):
+        """
+        Sends get stage instance request.
+
+        :param channel_id: ID of the channel.
+        """
+        return self.request(f"/stage-instances/{channel_id}", "GET")
+
+    def modify_stage_instance(self, channel_id, topic: str = None, privacy_level: int = None, reason: str = None):
+        """
+        Sends modify stage instance request.
+
+        :param channel_id: ID of the stage channel.
+        :param topic: Topic to change.
+        :param privacy_level: Privacy level to change.
+        :param reason: Reason of the action.
+        """
+        body = {}
+        if topic is not None:
+            body["topic"] = topic
+        if privacy_level is not None:
+            body["privacy_level"] = privacy_level
+        return self.request(f"/stage-instances/{channel_id}", "PATCH", body, is_json=True, reason_header=reason)
+
+    def delete_stage_instance(self, channel_id, reason: str = None):
+        """
+        Sends delete stage instance request.
+
+        :param channel_id: ID of the stage channel.
+        :param reason: Reason of the action.
+        """
+        return self.request(f"/stage-instances/{channel_id}", "DELETE", reason_header=reason)
+
+    # User Requests
+
+    def request_user(self, user_id="@me"):
+        """
+        Sends get user request.
+
+        :param user_id: ID of the user to get.
+        """
+        return self.request(f"/users/{user_id}", "GET")
+
+    def modify_current_user(self, username: str = None, avatar: str = EmptyObject):
+        """
+        Sends modify current user request. You can create avatar using ``to_image_data`` of utils.
+
+        :param username: Name to modify.
+        :param avatar: Image data to modify.
+        """
+        body = {}
+        if username is not None:
+            body["username"] = username
+        if avatar is not EmptyObject:
+            body["avatar"] = avatar
+        return self.request("/users/@me", "PATCH", body, is_json=True)
+
+    def request_current_user_guilds(self):
+        """
+        Sends get current user guilds request.
+        """
+        return self.request(f"/users/@me/guilds", "GET")
+
+    def leave_guild(self, guild_id):
+        """
+        Sends leave guild request.
+
+        :param guild_id: ID of the guild to leave.
+        """
+        return self.request(f"/users/@me/guilds/{guild_id}", "DELETE")
+
+    def create_dm(self, recipient_id: str):
+        """
+        Sends create dm request.
+
+        :param recipient_id: ID of the recipient to add.
+        """
+        return self.request(f"/users/@me/channels", "POST", {"recipient_id": recipient_id}, is_json=True)
+
+    def create_group_dm(self, access_tokens: typing.List[str], nicks: typing.Dict[str, str]):
+        """
+        Sends create group dm request.
+
+        :param access_tokens: OAuth2 access tokens of the participants to add.
+        :param nicks: Dict of ID-nick to set.
+        """
+        return self.request(f"/users/@me/channels", "POST", {"access_tokens": access_tokens, "nicks": nicks}, is_json=True)
+
+    # Voice Requests
+
+    def list_voice_regions(self):
+        """
+        Sends list voice regions request.
+        """
+        return self.request("/voice/regions", "GET")
+
     # Webhook Requests
 
     def create_webhook(self, channel_id, name: str, avatar: str = None):
@@ -1909,6 +2013,14 @@ class HTTPRequestBase(ABC):
         """
         return self.request(f"/applications/{application_id}/guilds/{guild_id}/commands/permissions", "PUT", permissions, is_json=True)
 
+    # OAuth2 Requests
+
+    def request_current_bot_application_information(self):
+        """
+        Sends get current bot application information request.
+        """
+        return self.request("/oauth2/applications/@me", "GET")
+
     # Misc
 
     @abstractmethod
