@@ -25,10 +25,13 @@ class EventBase:
 
 class DiscordObjectBase:
     TYPING = typing.Union[int, str, Snowflake, "DiscordObjectBase"]
+    RESPONSE = typing.Union["DiscordObjectBase", typing.Awaitable["DiscordObjectBase"]]
+    RESPONSE_AS_LIST = typing.Union[typing.List["DiscordObjectBase"], typing.Awaitable[typing.List["DiscordObjectBase"]]]
+    _cache_type = None
 
     def __init__(self, client, resp, **kwargs):
         resp.update(kwargs)
-        self._cache_type = None
+        # self._cache_type = None
         self.raw: dict = resp
         self.id: Snowflake = Snowflake(resp["id"])
         self.client: APIClient = client
@@ -38,7 +41,8 @@ class DiscordObjectBase:
 
     @classmethod
     def create(cls, client, resp, **kwargs):
-        maybe_exist = client.has_cache and client.cache.get(resp["id"], kwargs.pop("ensure_cache_type") if "ensure_cache_type" in kwargs else None)
+        ensure_cache_type = kwargs.pop("ensure_cache_type", cls._cache_type)
+        maybe_exist = client.has_cache and client.cache.get(resp["id"], ensure_cache_type)
         if maybe_exist:
             orig = maybe_exist.raw
             for k, v in resp.items():
@@ -56,6 +60,9 @@ class DiscordObjectBase:
 
 
 class AbstractObject(dict):
+    RESPONSE = typing.Union["AbstractObject", typing.Awaitable["AbstractObject"]]
+    RESPONSE_AS_LIST = typing.Union[typing.List["AbstractObject"], typing.Awaitable[typing.List["AbstractObject"]]]
+
     def __init__(self, resp):
         super().__init__(**resp)
 
