@@ -11,7 +11,7 @@ from .ws.websocket import WebSocketClient
 from .cache import CacheContainer
 from .exception import WebsocketClosed
 from .handler import EventHandler
-from .model import Intents, AllowedMentions, Snowflake, Application, Activity, Guild, Channel
+from .model import Intents, AllowedMentions, Snowflake, Activity, Guild, Channel
 
 
 class Client(APIClient):
@@ -41,19 +41,21 @@ class Client(APIClient):
                  default_allowed_mentions: AllowedMentions = None,
                  loop=None,
                  cache: bool = True,
-                 application_id: typing.Union[str, int, Snowflake] = None):
+                 application_id: typing.Union[str, int, Snowflake] = None,
+                 **cache_max_sizes):
+        cache_max_sizes.setdefault("message", 1000)
         self.loop = loop or asyncio.get_event_loop()
         super().__init__(token, base=AsyncHTTPRequest, default_allowed_mentions=default_allowed_mentions, loop=loop)
         self.token = token
         self.__use_cache = cache
-        self.cache = CacheContainer() if self.__use_cache else None  # The reason only supporting caching with WS is to ensure up-to-date object.
+        self.cache = CacheContainer(**cache_max_sizes) if self.__use_cache else None  # The reason only supporting caching with WS is to ensure up-to-date object.
         self.__ws_class = WebSocketClient
         self.intents = intents
         self.ws: typing.Union[None, WebSocketClient] = None
         self.events = EventHandler(self)
-        self.application: typing.Union[None, Application] = None
         self.__wait_futures = {}
         self.application_id = Snowflake.ensure_snowflake(application_id)
+        self.__ready_future = asyncio.Future()
 
         # Internal events dispatch
         self.events.add("READY", self.__ready)
@@ -61,6 +63,8 @@ class Client(APIClient):
 
     def __ready(self, ready):
         self.application_id = Snowflake(ready.application["id"])
+        if not self.__ready_future.done():
+            self.__ready_future.set_result(True)
 
     def __voice_state_update(self, voice_state):
         if self.has_cache:
@@ -127,30 +131,25 @@ class Client(APIClient):
                 if not fut.cancelled():
                     fut.set_result(args)
 
+    async def wait_ready(self):
+        if not self.__ready_future.done():
+            await self.__ready_future
+
     @property
     def get(self):
         """Alias of ``.cache.get``."""
         if self.has_cache:
             return self.cache.get
 
-    async def start(self, reconnect_on_unknown_disconnect: bool = False):
+    async def start(self, reconnect_on_unknown_disconnect: bool = False, compress: bool = True):
         """
-        Starts websocket connection and clears every connections after stopping due to error or KeyboardInterrupt.
+        Starts websocket connection.
 
         .. warning::
             You can call this only once.
         """
-        self.ws = await self.__ws_class.connect(self.http, self.intents, self.events, reconnect_on_unknown_disconnect)
-        try:
-            await self.ws.run()
-        except KeyboardInterrupt:
-            print("Detected KeyboardInterrupt, exiting...", file=sys.stderr)
-        except Exception as ex:
-            print("Unexpected exception occurred, exiting...", file=sys.stderr)
-            traceback.print_exc()
-        finally:
-            await self.ws.close()
-            await self.http.close()
+        self.ws = await self.__ws_class.connect(self.http, self.intents, self.events, reconnect_on_unknown_disconnect, compress)
+        await self.ws.run()
 
     async def close(self):
         """Clears all connections and closes session."""
@@ -200,6 +199,10 @@ class Client(APIClient):
             return self.ws.closed
         return True
 
+    @property
+    def guild_count(self):
+        return self.cache.get_size("guild")
+
     def __setattr__(self, key, value):
         if not key.lower().startswith("on_") or key.lower() in ["on", "on_"]:
             return super().__setattr__(key, value)
@@ -207,8 +210,14 @@ class Client(APIClient):
         return self.on_(event_name, value)
 
     def __getattr__(self, item):
-        if not item.lower().startswith("on_") or item.lower() in ["on", "on_"]:
+        if item.startswith("get_"):
+            def wrap(snowflake_id):
+                return self.get(snowflake_id, item[4:])  # noqa
+            return wrap
+
+        if not item.lower().startswith("on_") or item.lower() in ["on", "on_", "get"]:
             return super().__getattribute__(item)  # Should raise AttributeError or whatever.
+
         event_name = item.lower().lstrip("on_")
 
         def deco(func):
@@ -216,11 +225,19 @@ class Client(APIClient):
 
         return deco
 
-    def run(self, reconnect_on_unknown_disconnect: bool = False):
+    def run(self, *, reconnect_on_unknown_disconnect: bool = False, compress: bool = False):
         """
-        Runs client. Actually this is sync function wrapping :meth:`.start`.
+        Runs client and clears every connections after stopping due to error or KeyboardInterrupt.
 
         .. warning::
             This must be placed at the end of the code.
         """
-        self.loop.run_until_complete(self.start(reconnect_on_unknown_disconnect))
+        try:
+            self.loop.run_until_complete(self.start(reconnect_on_unknown_disconnect, compress))
+        except KeyboardInterrupt:
+            print("Detected KeyboardInterrupt, exiting...", file=sys.stderr)
+        except Exception as ex:
+            print("Unexpected exception occurred, exiting...", file=sys.stderr)
+            traceback.print_exc()
+        finally:
+            self.loop.run_until_complete(self.close())
