@@ -5,7 +5,7 @@ import logging
 import asyncio
 import aiohttp
 from .ratelimit import RatelimitHandler
-from .. import exception
+from .. import exception, __version__
 from ..base.http import HTTPRequestBase, EmptyObject
 
 
@@ -67,7 +67,7 @@ class AsyncHTTPRequest(HTTPRequestBase):
                 wait_time = (locker["reset_at"] - self.ratelimits.utc).total_seconds()
                 self.logger.warning(f"No more remaining request count, waiting for {wait_time} seconds...")
                 await asyncio.sleep(wait_time)
-            headers = {"Authorization": f"Bot {self.token}"}
+            headers = {"Authorization": f"Bot {self.token}", "User-Agent": f"DiscordBot (https://github.com/dico-api/dico, {__version__})"}
             if meth not in ["GET"] and body is not None:
                 if is_json:
                     headers["Content-Type"] = "application/json"
@@ -198,7 +198,7 @@ class AsyncHTTPRequest(HTTPRequestBase):
             payload_json["flags"] = flags
         params = {}
         if wait is not None:
-            params["wait"] = wait
+            params["wait"] = "true" if wait else "false"
         if thread_id is not None:
             params["thread_id"] = thread_id
         form.add_field("payload_json", json.dumps(payload_json), content_type="application/json")
