@@ -19,10 +19,12 @@ if typing.TYPE_CHECKING:
 
 class Channel(DiscordObjectBase):
     TYPING = typing.Union[int, str, Snowflake, "Channel"]
+    RESPONSE = typing.Union["Channel", typing.Awaitable["Channel"]]
+    RESPONSE_AS_LIST = typing.Union[typing.List["Channel"], typing.Awaitable[typing.List["Channel"]]]
+    _cache_type = "channel"
 
     def __init__(self, client, resp, *, guild_id=None):
         super().__init__(client, resp)
-        self._cache_type = "channel"
         self.type: ChannelTypes = ChannelTypes(resp["type"])
         self.guild_id: Snowflake = Snowflake.optional(resp.get("guild_id")) or Snowflake.ensure_snowflake(guild_id)
         self.position: typing.Optional[int] = resp.get("position")
@@ -52,7 +54,9 @@ class Channel(DiscordObjectBase):
         self.__permissions = resp.get("permissions")
         self.permissions: typing.Optional[PermissionFlags] = PermissionFlags.from_value(int(self.__permissions)) if self.__permissions else self.__permissions
 
-    def modify(self, **kwargs):
+        # if self.type.dm and self.
+
+    def modify(self, **kwargs) -> RESPONSE:
         if self.type.group_dm:
             return self.client.modify_group_dm_channel(self.id, **kwargs)
         elif self.type.dm or self.type.guild_store:
@@ -66,7 +70,7 @@ class Channel(DiscordObjectBase):
     def edit(self):
         return self.modify
 
-    def delete(self, *, reason: str = None):
+    def delete(self, *, reason: str = None) -> RESPONSE:
         return self.client.delete_channel(self, reason=reason)
 
     def create_message(self, *args, **kwargs) -> typing.Union["Message", typing.Awaitable["Message"]]:
@@ -205,13 +209,27 @@ class VideoQualityModes(TypeBase):
     FULL = 2
 
 
+class SendOnlyChannel:
+    def __init__(self, client, channel_id):
+        self.client = client
+        self.id: Snowflake = Snowflake.ensure_snowflake(channel_id)
+
+    def send(self, *args, **kwargs) -> "Message.RESPONSE":
+        return self.client.create_message(self.id, *args, **kwargs)
+
+    def __getattr__(self, item):
+        return None
+
+
 class Message(DiscordObjectBase):
     TYPING = typing.Union[int, str, Snowflake, "Message"]
+    RESPONSE = typing.Union["Message", typing.Awaitable["Message"]]
+    RESPONSE_AS_LIST = typing.Union[typing.List["Message"], typing.Awaitable[typing.List["Message"]]]
+    _cache_type = "message"
 
     def __init__(self, client, resp, *, guild_id=None, webhook_token=None, interaction_token=None, original_response=False):
         from .interactions import MessageInteraction, Component  # Prevent circular import.
         super().__init__(client, resp)
-        self._cache_type = "message"
         self.channel_id: Snowflake = Snowflake(resp["channel_id"])
         self.guild_id: typing.Optional[Snowflake] = Snowflake.optional(resp.get("guild_id") or guild_id)
         self.author: User = User.create(client, resp["author"])
@@ -305,12 +323,16 @@ class Message(DiscordObjectBase):
             return self.client.get(self.guild_id, "guild")
 
     @property
-    def channel(self) -> typing.Optional[Channel]:
+    def channel(self) -> typing.Union[Channel, SendOnlyChannel]:
+        send_only = SendOnlyChannel(self.client, self.channel_id)
         if self.channel_id and self.client.has_cache:
             if self.guild_id:
-                return self.guild.get(self.channel_id, "channel") or self.client.get(self.channel_id, "channel")
+                return self.guild.get(self.channel_id, "channel") or self.client.get(self.channel_id, "channel") or send_only
             else:
-                return self.client.get(self.channel_id, "channel")
+                return self.client.get(self.channel_id, "channel") or send_only
+        elif self.author and self.client.has_cache:
+            return self.client.get(self.author.dm_channel_id, "channel") or send_only
+        return send_only
 
 
 class MessageTypes(TypeBase):
@@ -395,6 +417,8 @@ class MessageReference:
 
 
 class FollowedChannel:
+    RESPONSE = typing.Union["FollowedChannel", typing.Awaitable["FollowedChannel"]]
+
     def __init__(self, client, resp):
         self.client: "APIClient" = client
         self.channel_id: typing.Optional[Snowflake] = Snowflake(resp["channel_id"])
@@ -464,6 +488,9 @@ class ThreadMetadata:
 
 
 class ThreadMember:
+    RESPONSE = typing.Union["ThreadMember", typing.Awaitable["ThreadMember"]]
+    RESPONSE_AS_LIST = typing.Union[typing.List["ThreadMember"], typing.Awaitable[typing.List["ThreadMember"]]]
+
     def __init__(self, client, resp):
         self.client: APIClient = client
         self.id: typing.Optional[Snowflake] = Snowflake.optional(resp.get("id"))
@@ -802,6 +829,8 @@ class AllowedMentions(CopyableObject):
 
 
 class ListThreadsResponse:
+    RESPONSE = typing.Union["ListThreadsResponse", typing.Awaitable["ListThreadsResponse"]]
+
     def __init__(self, client, resp):
         self.threads: typing.List[Channel] = [Channel.create(client, x) for x in resp["threads"]]
         self.members: typing.List[ThreadMember] = [ThreadMember(client, x) for x in resp["members"]]
