@@ -3,13 +3,14 @@ import datetime
 from .channel import Channel, Message, ThreadMember
 from .emoji import Emoji
 from .gateway import Activity
-from .guild import Guild, GuildMember
+from .guild import Guild, GuildMember, Integration
 from .permission import Role
 from .snowflake import Snowflake
 from .stage import StageInstance
+from .sticker import Sticker
 from .user import User
 from .voice import VoiceState
-from .interactions import Interaction, ApplicationCommand, ApplicationCommandOption
+from .interactions import Interaction  # , ApplicationCommand, ApplicationCommandOption
 from ..base.model import EventBase
 
 
@@ -24,6 +25,7 @@ class Ready(EventBase):
         self.application = resp["application"]
 
 
+"""
 class ApplicationCommandCreate(ApplicationCommand):
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
@@ -33,13 +35,14 @@ class ApplicationCommandCreate(ApplicationCommand):
     @classmethod
     def create(cls, client, resp):  # noqa
         resp["client"] = client
-        resp["options"] = [ApplicationCommandOption.create(x) for x in resp.get("options", [])]
+        resp["options"] = [ApplicationCommandOption.create(x) for x in resp.pop("options", [])]
+        resp["command_type"] = resp.pop("type", 1)
         return cls(**resp)
 
 
 ApplicationCommandUpdate = ApplicationCommandCreate
 ApplicationCommandDelete = ApplicationCommandCreate
-
+"""
 
 ChannelCreate = Channel
 
@@ -130,9 +133,31 @@ class ThreadMembersUpdate(EventBase):
             return self.client.get(self.guild_id, "guild")
 
 
-GuildCreate = Guild
-GuildUpdate = Guild
-GuildDelete = Guild
+class GuildCreate(Guild):
+    @classmethod
+    def create(cls, client, resp, **kwargs):
+        kwargs.setdefault("ensure_cache_type", "guild")
+        return super().create(client, resp, **kwargs)
+
+
+class GuildUpdate(Guild):
+    def __del__(self):
+        Guild.create(self.client, self.raw, ensure_cache_type="guild")
+
+    @classmethod
+    def create(cls, client, resp, **kwargs):
+        return cls(client, resp)
+
+    @property
+    def original(self):
+        if self.client.has_cache:
+            return self.client.get(self.id, "guild")
+
+
+class GuildDelete(GuildCreate):
+    def __del__(self):
+        if self.client.has_cache:
+            self.client.cache.remove(self.id, self._cache_type)
 
 
 class GuildBanAdd(EventBase):
@@ -163,6 +188,18 @@ class GuildEmojisUpdate(EventBase):
             return self.client.get(self.guild_id, "guild")
 
 
+class GuildStickersUpdate(EventBase):
+    def __init__(self, client, resp: dict):
+        super().__init__(client, resp)
+        self.guild_id = Snowflake(resp["guild_id"])
+        self.stickers = [Sticker.create(client, x) for x in resp["stickers"]]
+
+    @property
+    def guild(self):
+        if self.client.has_cache:
+            return self.client.get(self.guild_id, "guild")
+
+
 class GuildIntegrationsUpdate(EventBase):
     def __init__(self, client, resp: dict):
         super().__init__(client, resp)
@@ -265,6 +302,37 @@ class GuildRoleDelete(EventBase):
             return self.guild.get(self.role_id, "role")
 
 
+class IntegrationCreate(Integration):
+    def __init__(self, client, resp: dict):
+        super().__init__(client, resp)
+        self.guild_id: Snowflake = Snowflake(resp["guild_id"])
+
+    @property
+    def guild(self):
+        if self.client.has_cache:
+            return self.client.cache.get(self.guild_id, "guild")
+
+    @classmethod
+    def create(cls, client, resp: dict):
+        return cls(client, resp)
+
+
+IntegrationUpdate = IntegrationCreate
+
+
+class IntegrationDelete(EventBase):
+    def __init__(self, client, resp: dict):
+        super().__init__(client, resp)
+        self.id: Snowflake = Snowflake(resp["id"])
+        self.guild_id: Snowflake = Snowflake(resp["guild_id"])
+        self.application_id: typing.Optional[Snowflake] = Snowflake.optional(resp.get("application_id"))
+
+    @property
+    def guild(self):
+        if self.client.has_cache:
+            return self.client.cache.get(self.guild_id, "guild")
+
+
 InteractionCreate = Interaction
 
 
@@ -330,7 +398,7 @@ class MessageUpdate(Message):
             return cls(client, resp, **kwargs)
         except KeyError:
             if client.has_cache:
-                msg = client.get(resp["id"])
+                msg = client.get(resp["id"], cls._cache_type)
                 if msg:
                     orig = msg.raw
                     for k, v in resp.items():
@@ -514,7 +582,7 @@ class PresenceUpdate(EventBase):
     def __init__(self, client, resp: dict):
         super().__init__(client, resp)
         self.user = User.create(self.client, resp["user"])
-        self.guild_id = Snowflake(resp["guild_id"])
+        self.guild_id = Snowflake.optional(resp.get("guild_id"))
         self.status = resp["status"]
         self.activities = [Activity(x) for x in resp["activities"]]
         self.client_status = resp["client_status"]
@@ -587,3 +655,33 @@ class UserUpdate(User):
 
 
 VoiceStateUpdate = VoiceState
+
+
+class VoiceServerUpdate(EventBase):
+    def __init__(self, client, resp: dict):
+        super().__init__(client, resp)
+        self.token: str = resp["token"]
+        self.guild_id: Snowflake = Snowflake(resp["guild_id"])
+        self.endpoint: typing.Optional[str] = resp.get("endpoint")
+
+    @property
+    def guild(self):
+        if self.client.has_cache:
+            return self.client.get(self.guild_id, "guild")
+
+
+class WebhooksUpdate(EventBase):
+    def __init__(self, client, resp: dict):
+        super().__init__(client, resp)
+        self.guild_id: Snowflake = Snowflake(resp["guild_id"])
+        self.channel_id: Snowflake = Snowflake(resp["channel_id"])
+
+    @property
+    def guild(self):
+        if self.client.has_cache:
+            return self.client.get(self.guild_id, "guild")
+
+    @property
+    def channel(self):
+        if self.client.has_cache:
+            return self.client.get(self.channel_id, "channel")
