@@ -4,6 +4,8 @@ import datetime
 from .emoji import Emoji
 from .permission import Role, PermissionFlags
 from .snowflake import Snowflake
+from .stage import StageInstance
+from .sticker import Sticker
 from .user import User
 from ..utils import cdn_url
 from ..base.model import DiscordObjectBase, TypeBase, FlagBase
@@ -15,78 +17,86 @@ if typing.TYPE_CHECKING:
 
 class Guild(DiscordObjectBase):
     TYPING = typing.Union[int, str, Snowflake, "Guild"]
+    RESPONSE = typing.Union["Guild", typing.Awaitable["Guild"]]
+    RESPONSE_AS_LIST = typing.Union[typing.List["Guild"], typing.Awaitable[typing.List["Guild"]]]
+    _cache_type = "guild"
 
     def __init__(self, client, resp):
         from .channel import Channel  # Prevent circular import.
+        from .event import PresenceUpdate
+        from .voice import VoiceState
         super().__init__(client, resp)
-        self._cache_type = "guild"
-        self.name = resp["name"]
-        self.icon = resp["icon"]
-        self.icon_hash = resp.get("icon_hash")
-        self.splash = resp["splash"]
-        self.discovery_splash = resp["discovery_splash"]
-        self.owner = resp.get("owner", False)
-        self.owner_id = Snowflake(resp["owner_id"])
-        self.permissions = resp.get("permissions")
-        self.region = resp["region"]
-        self.afk_channel_id = Snowflake.optional(resp["afk_channel_id"])
-        self.afk_timeout = resp["afk_timeout"]
-        self.widget_enabled = resp.get("widget_enabled")
-        self.widget_channel_id = Snowflake.optional(resp.get("widget_channel_id"))
-        self.verification_level = VerificationLevel(resp["verification_level"])
-        self.default_message_notifications = DefaultMessageNotificationLevel(resp["default_message_notifications"])
-        self.explicit_content_filter = ExplicitContentFilterLevel(resp["explicit_content_filter"])
-        self.roles = [Role.create(client, x, guild_id=self.id) for x in resp["roles"]]
-        self.emojis = [Emoji(self.client, x) for x in resp["emojis"]]
-        self.features = resp["features"]
-        self.mfa_level = MFALevel(resp["mfa_level"])
-        self.application_id = Snowflake.optional(resp["application_id"])
-        self.system_channel_id = Snowflake.optional(resp["system_channel_id"])
-        self.system_channel_flags = SystemChannelFlags.from_value(resp["system_channel_flags"])
-        self.rules_channel_id = Snowflake.optional(resp["rules_channel_id"])
-        self.__joined_at = resp["joined_at"]
-        self.joined_at = datetime.datetime.fromisoformat(self.__joined_at) if self.__joined_at else self.__joined_at
-        self.large = resp.get("large", False)
-        self.unavailable = resp.get("unavailable", False)
-        self.member_count = resp.get("member_count", 0)
-        self.voice_states = resp.get("voice_states", [])
-        self.members = [GuildMember.create(self.client, x, guild_id=self.id) for x in resp.get("members", [])]
-        self.channels = [Channel.create(client, x, guild_id=self.id) for x in resp.get("channels", [])]
-        self.presences = resp.get("presences", [])
-        self.max_presences = resp.get("max_presences", 25000)
-        self.max_members = resp.get("max_members")
-        self.vanity_url_code = resp["vanity_url_code"]
-        self.description = resp["description"]
-        self.banner = resp["banner"]
-        self.premium_tier = PremiumTier(resp["premium_tier"])
-        self.premium_subscription_count = resp.get("premium_subscription_count", 0)
-        self.preferred_locale = resp["preferred_locale"]
-        self.public_updates_channel_id = Snowflake.optional(resp["public_updates_channel_id"])
-        self.max_video_channel_users = resp.get("max_video_channel_users")
-        self.approximate_member_count = resp.get("approximate_member_count")
-        self.approximate_presence_count = resp.get("approximate_presence_count")
-        self.welcome_screen = resp.get("welcome_screen")
-        self.nsfw = resp["nsfw"]
+        self.name: str = resp["name"]
+        self.icon: typing.Optional[str] = resp["icon"]
+        self.icon_hash: typing.Optional[str] = resp.get("icon_hash")
+        self.splash: typing.Optional[str] = resp["splash"]
+        self.discovery_splash: typing.Optional[str] = resp["discovery_splash"]
+        self.owner: typing.Optional[bool] = resp.get("owner")
+        self.owner_id: Snowflake = Snowflake(resp["owner_id"])
+        self.permissions: typing.Optional[str] = resp.get("permissions")
+        self.region: typing.Optional[str] = resp["region"]
+        self.afk_channel_id: typing.Optional[Snowflake] = Snowflake.optional(resp["afk_channel_id"])
+        self.afk_timeout: int = resp["afk_timeout"]
+        self.widget_enabled: typing.Optional[bool] = resp.get("widget_enabled")
+        self.widget_channel_id: typing.Optional[Snowflake] = Snowflake.optional(resp.get("widget_channel_id"))
+        self.verification_level: "VerificationLevel" = VerificationLevel(resp["verification_level"])
+        self.default_message_notifications: "DefaultMessageNotificationLevel" = DefaultMessageNotificationLevel(resp["default_message_notifications"])
+        self.explicit_content_filter: "ExplicitContentFilterLevel" = ExplicitContentFilterLevel(resp["explicit_content_filter"])
+        self.roles: typing.List[Role] = [Role.create(client, x, guild_id=self.id) for x in resp["roles"]]
+        self.emojis: typing.List[Emoji] = [Emoji(self.client, x) for x in resp["emojis"]]
+        self.features: typing.List[str] = resp["features"]
+        self.mfa_level: "MFALevel" = MFALevel(resp["mfa_level"])
+        self.application_id: typing.Optional[Snowflake] = Snowflake.optional(resp["application_id"])
+        self.system_channel_id: typing.Optional[Snowflake] = Snowflake.optional(resp["system_channel_id"])
+        self.system_channel_flags: SystemChannelFlags = SystemChannelFlags.from_value(resp["system_channel_flags"])
+        self.rules_channel_id: typing.Optional[Snowflake] = Snowflake.optional(resp["rules_channel_id"])
+        self.__joined_at = resp.get("joined_at")
+        self.joined_at: typing.Optional[datetime.datetime] = datetime.datetime.fromisoformat(self.__joined_at) if self.__joined_at else self.__joined_at
+        self.large: typing.Optional[bool] = resp.get("large", False)
+        self.unavailable: typing.Optional[bool] = resp.get("unavailable", False)
+        self.member_count: typing.Optional[int] = resp.get("member_count", 0)
+        self.voice_states: typing.Optional[typing.List[VoiceState]] = [VoiceState.create(client, x) for x in resp.get("voice_states", [])]
+        self.members: typing.Optional[typing.List["GuildMember"]] = [GuildMember.create(self.client, x, guild_id=self.id) for x in resp.get("members", [])]
+        self.channels: typing.Optional[typing.List[Channel]] = [Channel.create(client, x, guild_id=self.id) for x in resp.get("channels", [])]
+        self.threads: typing.Optional[typing.List[Channel]] = [Channel.create(client, x, guild_id=self.id, ensure_cache_type="channel") for x in resp.get("threads", [])]
+        self.presences: typing.Optional[typing.List[PresenceUpdate]] = [PresenceUpdate.create(client, x) for x in resp.get("presences", [])]
+        self.max_presences: typing.Optional[int] = resp.get("max_presences")
+        self.max_members: typing.Optional[int] = resp.get("max_members")
+        self.vanity_url_code: typing.Optional[str] = resp["vanity_url_code"]
+        self.description: typing.Optional[str] = resp["description"]
+        self.banner: typing.Optional[str] = resp["banner"]
+        self.premium_tier: "PremiumTier" = PremiumTier(resp["premium_tier"])
+        self.premium_subscription_count: int = resp.get("premium_subscription_count", 0)
+        self.preferred_locale: str = resp["preferred_locale"]
+        self.public_updates_channel_id: typing.Optional[Snowflake] = Snowflake.optional(resp["public_updates_channel_id"])
+        self.max_video_channel_users: typing.Optional[int] = resp.get("max_video_channel_users")
+        self.approximate_member_count: typing.Optional[int] = resp.get("approximate_member_count")
+        self.approximate_presence_count: typing.Optional[int] = resp.get("approximate_presence_count")
+        self.__welcome_screen = resp.get("welcome_screen")
+        self.welcome_screen: typing.Optional["WelcomeScreen"] = WelcomeScreen(self.__welcome_screen) if self.__welcome_screen else self.__welcome_screen
+        self.nsfw_level: "NSFWLevel" = NSFWLevel(resp["nsfw_level"])
+        self.stage_instances: typing.Optional[typing.List[StageInstance]] = [StageInstance.create(client, x) for x in resp.get("stage_instances", [])]
+        self.stickers: typing.Optional[typing.List[Sticker]] = [Sticker.create(client, x) for x in resp.get("stickers", [])]
 
         self.cache = client.cache.get_guild_container(self.id) if client.has_cache else None
 
-    def icon_url(self, *, extension="webp", size=1024):
+    def icon_url(self, *, extension="webp", size=1024) -> typing.Optional[str]:
         if self.icon:
             return cdn_url("icons/{guild_id}", image_hash=self.icon, extension=extension, size=size, guild_id=self.id)
 
-    def splash_url(self, *, extension="webp", size=1024):
+    def splash_url(self, *, extension="webp", size=1024) -> typing.Optional[str]:
         if self.splash:
             return cdn_url("splashes/{guild_id}", image_hash=self.splash, extension=extension, size=size, guild_id=self.id)
 
-    def discovery_splash_url(self, *, extension="webp", size=1024):
+    def discovery_splash_url(self, *, extension="webp", size=1024) -> typing.Optional[str]:
         if self.discovery_splash:
             return cdn_url("discovery-splashes/{guild_id}", image_hash=self.discovery_splash, extension=extension, size=size, guild_id=self.id)
 
-    def banner_url(self, *, extension="webp", size=1024):
+    def banner_url(self, *, extension="webp", size=1024) -> typing.Optional[str]:
         if self.banner:
             return cdn_url("banners/{guild_id}", image_hash=self.banner, extension=extension, size=size, guild_id=self.id)
 
-    def request_preview(self):
+    def request_preview(self) -> "GuildPreview.RESPONSE":
         return self.client.request_guild_preview(self)
 
     def delete(self):
@@ -99,22 +109,22 @@ class Guild(DiscordObjectBase):
     def edit(self):
         return self.modify
 
-    def request_channels(self):
+    def request_channels(self) -> typing.Union[typing.List["Channel"], typing.Awaitable[typing.List["Channel"]]]:
         return self.client.request_guild_channels(self)
 
-    def create_channel(self, name: str, **kwargs):
+    def create_channel(self, name: str, **kwargs) -> "Channel.RESPONSE":
         return self.client.create_guild_channel(self, name, **kwargs)
 
     def modify_channel_positions(self, *params: dict, reason: str = None):
         return self.client.modify_guild_channel_positions(self, *params, reason=reason)
 
-    def list_active_threads(self):
+    def list_active_threads(self) -> typing.Union[typing.List["Channel"], typing.Awaitable[typing.List["Channel"]]]:
         return self.client.list_active_threads_as_guild(self)
 
-    def request_member(self, user: User.TYPING):
+    def request_member(self, user: User.TYPING) -> "GuildMember.RESPONSE":
         return self.client.request_guild_member(self, user)
 
-    def list_members(self, limit: int = None, after: str = None):
+    def list_members(self, limit: int = None, after: str = None) -> typing.Union[typing.List["GuildMember"], typing.Awaitable[typing.List["GuildMember"]]]:
         return self.client.list_guild_members(self, limit, after)
 
     def remove_guild_member(self, user: User.TYPING):
@@ -138,7 +148,7 @@ class Guild(DiscordObjectBase):
     def unban(self):
         return self.remove_ban
 
-    def request_roles(self):
+    def request_roles(self) -> typing.List[Role]:
         return self.client.request_guild_roles(self)
 
     def create_role(self,
@@ -148,7 +158,7 @@ class Guild(DiscordObjectBase):
                     color: int = None,
                     hoist: bool = None,
                     mentionable: bool = None,
-                    reason: str = None):
+                    reason: str = None) -> Role:
         kwargs = {"name": name,
                   "permissions": permissions,
                   "color": color,
@@ -242,6 +252,9 @@ class Guild(DiscordObjectBase):
                                 request_to_speak_timestamp: typing.Union[datetime.datetime, str] = None):
         return self.client.modify_user_voice_state(self, channel, user, suppress=suppress, request_to_speak_timestamp=request_to_speak_timestamp)
 
+    def leave(self):
+        return self.client.leave_guild(self)
+
     @property
     def get(self):
         """Alias of ``Guild.cache.get``."""
@@ -277,6 +290,13 @@ class VerificationLevel(TypeBase):
     VERY_HIGH = 4
 
 
+class NSFWLevel(TypeBase):
+    DEFAULT = 0
+    EXPLICIT = 1
+    SAFE = 2
+    AGE_RESTRICTED = 3
+
+
 class PremiumTier(TypeBase):
     NONE = 0
     TIER_1 = 1
@@ -291,6 +311,8 @@ class SystemChannelFlags(FlagBase):
 
 
 class GuildPreview:
+    RESPONSE = typing.Union["GuildPreview", typing.Awaitable["GuildPreview"]]
+
     def __init__(self, client, resp):
         self.id = Snowflake(resp["id"])
         self.name = resp["name"]
@@ -305,6 +327,9 @@ class GuildPreview:
 
 
 class GuildWidget:
+    RESPONSE = typing.Union["GuildWidget", typing.Awaitable["GuildWidget"]]
+    RESPONSE_AS_LIST = typing.Union[typing.List["GuildWidget"], typing.Awaitable[typing.List["GuildWidget"]]]
+
     def __init__(self, resp):
         self.enabled = resp["enabled"]
         self.channel_id = Snowflake.optional(resp["channel_id"])
@@ -312,6 +337,8 @@ class GuildWidget:
 
 class GuildMember:
     TYPING = typing.Union[int, str, Snowflake, "GuildMember"]
+    RESPONSE = typing.Union["GuildMember", typing.Awaitable["GuildMember"]]
+    RESPONSE_AS_LIST = typing.Union[typing.List["GuildMember"], typing.Awaitable[typing.List["GuildMember"]]]
 
     def __init__(self, client, resp, *, user: User = None, guild_id=None):
         self.raw = resp
@@ -320,7 +347,7 @@ class GuildMember:
         self.__user = resp.get("user")
         self.user = User.create(client, self.__user) if self.__user and not self.user else self.user if self.user else self.__user
         self.nick = resp.get("nick")
-        self.roles = [client.get(x) for x in resp["roles"]] if client.has_cache else [Snowflake(x) for x in resp["roles"]]
+        self.roles = [client.get(x, "role") for x in resp["roles"]] if client.has_cache else [Snowflake(x) for x in resp["roles"]]
         self.joined_at = datetime.datetime.fromisoformat(resp["joined_at"])
         self.__premium_since = resp.get("premium_since")
         self.premium_since = datetime.datetime.fromisoformat(self.__premium_since) if self.__premium_since else self.__premium_since
@@ -353,11 +380,17 @@ class GuildMember:
             return f"<@!{self.user.id}>"
 
     @property
-    def permissions(self):
+    def permissions(self) -> typing.Optional[PermissionFlags]:
         if self.__permissions:
             return PermissionFlags.from_value(int(self.__permissions))
         elif self.roles:
-            raise NotImplementedError
+            value = 0
+            for x in self.roles:
+                if isinstance(x, Role):
+                    value |= x.permissions.value
+                else:
+                    return None
+            return PermissionFlags.from_value(value)
         else:
             return PermissionFlags.from_value(0)
 
@@ -382,8 +415,12 @@ class GuildMember:
 
 class Integration:
     TYPING = typing.Union[int, str, Snowflake, "Integration"]
+    RESPONSE = typing.Union["Integration", typing.Awaitable["Integration"]]
+    RESPONSE_AS_LIST = typing.Union[typing.List["Integration"], typing.Awaitable[typing.List["Integration"]]]
 
     def __init__(self, client, resp):
+        self.client = client
+        self.raw = resp
         self.id = Snowflake(resp["id"])
         self.name = resp["name"]
         self.type = resp["type"]
@@ -407,6 +444,11 @@ class Integration:
     def __int__(self):
         return int(self.id)
 
+    @property
+    def role(self):
+        if self.client.has_cache:
+            return self.client.cache.get(self.role_id, "role")
+
 
 class IntegrationExpireBehaviors(TypeBase):
     REMOVE_ROLE = 0
@@ -434,12 +476,18 @@ class IntegrationApplication:
 
 
 class Ban:
+    RESPONSE = typing.Union["Ban", typing.Awaitable["Ban"]]
+    RESPONSE_AS_LIST = typing.Union[typing.List["Ban"], typing.Awaitable[typing.List["Ban"]]]
+
     def __init__(self, client, resp):
         self.reason = resp["reason"]
         self.user = User.create(client, resp["user"])
 
 
 class WelcomeScreen:
+    RESPONSE = typing.Union["WelcomeScreen", typing.Awaitable["WelcomeScreen"]]
+    RESPONSE_AS_LIST = typing.Union[typing.List["WelcomeScreen"], typing.Awaitable[typing.List["WelcomeScreen"]]]
+
     def __init__(self, resp):
         self.description = resp["description"]
         self.welcome_channels = [WelcomeScreenChannel(x) for x in resp["welcome_channels"]]
