@@ -1,11 +1,17 @@
+import typing
+
 from .snowflake import Snowflake
 from ..base.model import DiscordObjectBase, TypeBase
 
 
 class StageInstance(DiscordObjectBase):
+    TYPING = typing.Union[int, str, Snowflake, "StageInstance"]
+    RESPONSE = typing.Union["StageInstance", typing.Awaitable["StageInstance"]]
+    RESPONSE_AS_LIST = typing.Union[typing.List["StageInstance"], typing.Awaitable[typing.List["StageInstance"]]]
+    _cache_type = "stage_instance"
+
     def __init__(self, client, resp):
         super().__init__(client, resp)
-        self._cache_type = "stage_instance"
         self.guild_id = Snowflake(resp["guild_id"])
         self.channel_id = Snowflake(resp["channel_id"])
         self.topic = resp["topic"]
