@@ -1,3 +1,4 @@
+import typing
 import datetime
 from .guild import GuildMember
 from .snowflake import Snowflake
@@ -44,6 +45,9 @@ class VoiceState:
 
 
 class VoiceRegion:
+    RESPONSE = typing.Union["VoiceRegion", typing.Awaitable["VoiceRegion"]]
+    RESPONSE_AS_LIST = typing.Union[typing.List["VoiceRegion"], typing.Awaitable[typing.List["VoiceRegion"]]]
+
     def __init__(self, resp):
         self.id = resp["id"]
         self.name = resp["name"]
@@ -51,3 +55,32 @@ class VoiceRegion:
         self.optimal = resp["optimal"]
         self.deprecated = resp["deprecated"]
         self.custom = resp["custom"]
+
+
+class Opcodes:
+    IDENTIFY = 0
+    SELECT_PROTOCOL = 1
+    READY = 2
+    HEARTBEAT = 3
+    SESSION_DESCRIPTION = 4
+    SPEAKING = 5
+    HEARTBEAT_ACK = 6
+    RESUME = 7
+    HELLO = 8
+    RESUMED = 9
+    CLIENT_DISCONNECT = 13
+
+    @staticmethod
+    def as_string(code: int) -> str:
+        opcodes = {0: "Identify",
+                   1: "Select Protocol",
+                   2: "Ready",
+                   3: "Heartbeat",
+                   4: "Session Description",
+                   5: "Speaking",
+                   6: "Heartbeat ACK",
+                   7: "Resume",
+                   8: "Hello",
+                   9: "Resumed",
+                   13: "Client Disconnect"}
+        return opcodes.get(code)
